@@ -65,6 +65,7 @@ namespace pan
     public:
       DataImpl (bool unit_test=false, int cache_megs=10, DataIO * source=new DataIO());
       virtual ~DataImpl ();
+      virtual void save_state ();
 
     public:
       virtual ArticleCache& get_cache () { return _cache; }
@@ -165,7 +166,8 @@ namespace pan
     private: // implementation
 
       typedef std::map<Quark,std::string> descriptions_t;
-      descriptions_t _descriptions; // groupname -> description
+      mutable descriptions_t _descriptions; // groupname -> description
+      mutable bool _descriptions_loaded;
 
       typedef sorted_vector<Quark,true> unique_sorted_quarks_t;
       typedef sorted_vector<Quark,true> groups_t;
@@ -242,8 +244,9 @@ namespace pan
         const ReadGroup * read_group = find_read_group (g);
         return read_group ? read_group->find_server (s) : 0;
       }
-          
-      void load_group_descriptions (const DataIO&);
+
+      void ensure_descriptions_are_loaded () const;
+      void load_group_descriptions (const DataIO&) const;
       void save_group_descriptions (DataIO&) const;
 
       void load_group_xovers (const DataIO&);
