@@ -37,14 +37,13 @@ using namespace pan;
 
 namespace
 {
-  void find_parts (const StringView   & subj,
-                   const Quark        & group,
-                   int                  line_count,
-                   int                & part,
-                   int                & parts)
+  bool parse_multipart_subject (const StringView   & subj,
+                                int                & part,
+                                int                & parts,
+                                std::string        & no_part)
   {
-    const StringView gname (group.c_str());
-    part = parts = 0;
+    const char * numerator = 0;
+    const char * denominator = 0;
 
     const char * s (subj.begin());
     const char * pch (subj.end());
@@ -62,66 +61,80 @@ namespace
         continue;
 
       // N -> parts
-      parts = atoi (pch+1);
-        --pch;
+      denominator = pch+1;
+      --pch;
 
       // find the '[' of [n/N]
       while (s!=pch && isdigit(*pch))
         --pch;
       if (s==pch || (*pch!='(' && *pch!='[')) {
-        parts = 0;
+        denominator = 0;
         continue;
       }
 
-      // n -> part
-      part = atoi (pch+1);
+      // N -> part
+      numerator = pch+1;
+      char * numerator_end (0);
+      part = (int) strtol (numerator, &numerator_end, 10);
+      parts = atoi (denominator);
 
-      if (part > parts) { // false positive
+      if (part > parts) {
+        // false positive...
+        numerator = denominator = 0;
         part = parts = 0;
         continue;
       }
 
-      break;
+      no_part.assign (subj.str, numerator-subj.str);
+      no_part.append (numerator_end, (subj.str+subj.len)-numerator_end);
+      return true;
+    }
+
+    return false;
+  }
+
+  void find_parts (const StringView   & subj,
+                   const Quark        & group,
+                   int                  line_count,
+                   int                & part,
+                   int                & parts,
+                   std::string        & norm)
+  {
+    if (!parse_multipart_subject (subj, part, parts, norm))
+    {
+      part = parts = 0;
+      norm.assign (subj.str, subj.len);
     }
 
     /* if not a multipart yet, AND if it's a big message, AND
        it's either in one of the pictures/fan/sex groups or it
        has commonly-used image names in the subject, it's probably
        a single-part binary */
-    if (!parts
-         && (line_count>400)
-         && (
-             ((gname.strstr("binaries") ||
-               gname.strstr("fan") ||
-               gname.strstr("mag") ||
-               gname.strstr("sex")))
-             ||
-             ((subj.strstr(".jpg") || subj.strstr(".JPG") ||
-               subj.strstr(".jpeg") || subj.strstr(".JPEG") ||
-               subj.strstr(".gif") || subj.strstr(".GIF") ||
-               subj.strstr(".png") || subj.strstr(".PNG")))
-            ))
-      part = parts = 1;
-                                                                                                                                       
+    if (!parts && line_count>400) {
+      const StringView gname (group.c_str());
+      if (((gname.strstr("binaries") ||
+            gname.strstr("fan") ||
+            gname.strstr("mag") ||
+            gname.strstr("sex")))
+           ||
+          ((subj.strstr(".jpg") || subj.strstr(".JPG") ||
+            subj.strstr(".jpeg") || subj.strstr(".JPEG") ||
+            subj.strstr(".gif") || subj.strstr(".GIF") ||
+            subj.strstr(".png") || subj.strstr(".PNG"))))
+        part = parts = 1;
+    }
+
     /* but if it's starting the subject with "Re:" and doesn't
        have many lines, it's probably a followup to a part, rather
        than an actual part. */
     if (Article::has_reply_leader(subj) && line_count<100)
       part = parts = 0;
-                                                                                                                                       
+
     /* Subjects containing (0/N) aren't part of an N-part binary;
        they're text description articles that accompany the binary. */
     if (part == 0)
       parts = 0;
   }
-
-  std::string
-  normalize_subject_for_multiparts (const StringView& subject)
-  {
-    std::string s;
-    mime :: remove_multipart_part_from_subject (subject, s);
-    return s;
-  }
 }
 
 void
@@ -167,11 +180,8 @@ DataImpl :: xover_ref (const Quark& group)
   foreach_const (nodes_t, h->_nodes, it) {
     const Quark& mid (it->first);
     const Article * a (it->second->_article);
-    if (a != 0) {
-      const StringView subject (a->subject.c_str());
-      const std::string norm (normalize_subject_for_multiparts (subject));
-      workarea._subject_lookup.insert (std::pair<Quark,Quark>(Quark(norm),mid));
-    }
+    if (a != 0)
+      workarea._subject_lookup.insert (std::pair<Quark,Quark>(a->subject,mid));
   }
 
   // FIXME: this could possibly cause a memory problem if
@@ -234,20 +244,18 @@ DataImpl :: xover_add (const Quark         & server,
   ***/
 
   int part_index, part_count;
-  find_parts (subject, group, line_count, part_index, part_count);
+  std::string multipart_subject;
+  find_parts (subject, group, line_count, part_index, part_count, multipart_subject);
   Quark art_mid;
-  std::string norm;
 
   if (part_count > 1)
   {
-    mime :: remove_multipart_part_from_subject (subject, norm);
-
     StringView ref(references), parent;
     ref.pop_last_token (parent, ' ');
     parent.trim ();
 
     typedef XOverEntry::subject_to_mid_t::const_iterator cit;
-    const std::pair<cit,cit> range (workarea._subject_lookup.equal_range (norm));
+    const std::pair<cit,cit> range (workarea._subject_lookup.equal_range (multipart_subject));
     for (cit it(range.first), end(range.second); it!=end && art_mid.empty(); ++it) {
       const Quark& candidate_mid (it->second);
       const Article* candidate (h->find_article (candidate_mid));
@@ -263,7 +271,7 @@ DataImpl :: xover_add (const Quark         & server,
     art_mid = message_id;
 
     if (part_count > 1)
-      workarea._subject_lookup.insert(std::pair<Quark,Quark>(norm, art_mid));
+      workarea._subject_lookup.insert(std::pair<Quark,Quark>(multipart_subject, art_mid));
 
     // if we don't already have this article...
     if (!h->find_article (art_mid))
@@ -271,7 +279,7 @@ DataImpl :: xover_add (const Quark         & server,
       //std::cerr << LINE_ID << " We didn't have this article yet, so creating an instance..." << std::endl;
       Article& a (h->alloc_new_article());
       a.author = author;
-      a.subject = subject;
+      a.subject = multipart_subject;
       a.message_id = art_mid;
       a.is_binary = part_count >= 1;
       a.set_part_count (a.is_binary ? part_count : 1);
@@ -313,15 +321,6 @@ DataImpl :: xover_add (const Quark         & server,
     on_articles_changed (workarea._changed_batch, true);
     workarea._changed_batch.clear();
     workarea._batch_parts_size = 0;
-
-    XOverEntry::subject_to_mid_t& s2m (workarea._subject_lookup);
-    for (XOverEntry::subject_to_mid_t::iterator it(s2m.begin()), end(s2m.end()); it!=end; ) {
-      Article * article (h->find_article (it->second));
-      if (!article || (article->get_part_state() != Article::INCOMPLETE))
-        workarea._subject_lookup.erase (it++);
-      else
-        ++it;
-    }
   }
 
   return new_article;
