@@ -404,9 +404,6 @@ ArticleCache :: get_message_mem_stream (const Quark& mid) const
 GMimeMessage*
 ArticleCache :: get_message (const mid_sequence_t& mids)
 {
-   foreach_const (mid_sequence_t, mids, it)
-      pan_return_val_if_fail (this->contains(*it), 0);
-
    debug ("trying to get a message with " << mids.size() << " parts");
    GMimeMessage * retval = NULL;
 
@@ -416,25 +413,22 @@ ArticleCache :: get_message (const mid_sequence_t& mids)
    const bool in_memory (mids.size() <= 2u);
    foreach_const (mid_sequence_t, mids, it) {
       const Quark mid (*it);
-      debug ("first message is " << mid);
-      if (this->contains (*it)) {
-         GMimeStream * stream = in_memory
-            ? get_message_mem_stream (mid)
-            : get_message_file_stream (mid);
-         if (!stream)
-            break;
-         debug ("got a stream " << stream << " for " << mid);
-         streams.push_back (stream);
-      }
+      GMimeStream * stream (0);
+      if (this->contains (*it))
+        stream = in_memory
+          ? get_message_mem_stream (mid)
+          : get_message_file_stream (mid);
+      if (stream)
+        streams.push_back (stream);
    }
 
    // build the message
-   if (streams.size() == mids.size())
-      retval = mime :: construct_message (&streams.front(), streams.size());
+   if (!streams.empty())
+     retval = mime :: construct_message (&streams.front(), streams.size());
 
    // cleanup
-   for (streams_t::iterator it=streams.begin(), end=streams.end(); it!=end; ++it) 
-      g_object_unref (*it);
+   foreach (streams_t, streams, it)
+     g_object_unref (*it);
 
    debug ("returning " << retval);
    return retval;
