@@ -590,18 +590,27 @@ namespace
     g_assert (!text.empty());
     g_assert (pixbuf != 0);
 
-    std::string::size_type pos;
-    while (((pos=body.find(text))) != body.npos)
+    GtkTextTagTable * tags (gtk_text_buffer_get_tag_table (buffer));
+    GtkTextTag * url_tag (gtk_text_tag_table_lookup (tags, "url"));
+
+    const size_t n (text.size());
+    std::string::size_type pos (0);
+    while (((pos=body.find(text,pos))) != body.npos)
     {
       GtkTextIter begin;
       gtk_text_buffer_get_iter_at_mark (buffer, &begin, mark);
       gtk_text_iter_forward_chars (&begin, g_utf8_strlen(&body[0],pos));
-      GtkTextIter end = begin;
-      gtk_text_iter_forward_chars (&end, text.size());
-      gtk_text_buffer_delete (buffer, &begin, &end);
-      body.erase (pos, text.size());
-      gtk_text_buffer_insert_pixbuf (buffer, &end, pixbuf);
-      body.insert (pos, 1, '?'); // make body.size() match the textbuf's size
+
+      if (gtk_text_iter_has_tag (&begin, url_tag))
+        pos += n;
+      else {
+        GtkTextIter end (begin);
+        gtk_text_iter_forward_chars (&end, n);
+        gtk_text_buffer_delete (buffer, &begin, &end);
+        body.erase (pos, text.size());
+        gtk_text_buffer_insert_pixbuf (buffer, &end, pixbuf);
+        body.insert (pos, 1, '?'); // make body.size() match the textbuf's size
+      }
     }
   }
 
@@ -931,7 +940,7 @@ namespace
 void
 BodyPane :: set_text_from_message (GMimeMessage * message)
 {
-  const char * fallback_charset (_charset.c_str());
+  const char * fallback_charset (_charset.empty() ? 0 : _charset.c_str());
 
   // manditory headers...
   std::string s;
@@ -1069,6 +1078,14 @@ BodyPane :: clear ()
   refresh ();
 }
 
+void
+BodyPane :: select_all ()
+{
+  GtkTextIter start, end;
+  gtk_text_buffer_get_bounds (_buffer, &start, &end);
+  gtk_text_buffer_select_range (_buffer, &start, &end);
+}
+
 void
 BodyPane :: rot13_selected_text ()
 {
@@ -1196,6 +1213,8 @@ BodyPane :: BodyPane (Data& data, ArticleCache& cache, Prefs& prefs):
   _prefs (prefs),
   _data (data),
   _cache (cache),
+  _hscroll_visible (false),
+  _vscroll_visible (false),
   _message (0)
 {
   GtkWidget * vbox = gtk_vbox_new (false, PAD);
@@ -1366,6 +1385,41 @@ namespace
       s = header_to_utf8 (v, fallback_charset_1, fallback_charset_2);
     return s;
   }
+
+  struct ForeachPartData
+  {
+    std::string fallback_charset;
+    std::string body;
+  };
+
+  void get_utf8_body_foreach_part (GMimeObject *o, gpointer user_data)
+  {
+    if (GMIME_IS_MULTIPART(o))
+    {
+      g_mime_multipart_foreach (GMIME_MULTIPART(o),
+                                get_utf8_body_foreach_part,
+                                user_data);
+    }
+    else
+    {
+      GMimePart * part = GMIME_PART (o);
+      const GMimeContentType * type = g_mime_object_get_content_type (o);
+      const bool is_text (g_mime_content_type_is_type (type, "text", "*"));
+      if (is_text)
+      {
+        ForeachPartData *data (static_cast<ForeachPartData*>(user_data));
+        data->body += mime_part_to_utf8 (part, data->fallback_charset.c_str());
+      }
+    }
+  }
+
+  std::string get_utf8_body (GMimeMessage * source)
+  {
+    ForeachPartData tmp;
+    if (source)
+      g_mime_message_foreach_part (source, get_utf8_body_foreach_part, &tmp);
+    return tmp.body;
+  }
 }
 
 GMimeMessage*
@@ -1444,16 +1498,14 @@ BodyPane :: create_followup_or_reply (bool is_reply)
     GtkTextIter start, end;
     if (gtk_text_buffer_get_selection_bounds (_buffer, &start, &end))
     {
-      // go with the user-selected region w/o modifications
+      // go with the user-selected region w/o modifications.
+      // since it's in the text pane it's already utf-8...
       h = gtk_text_buffer_get_text (_buffer, &start, &end, false);
     }
     else
     {
       // get the entire body...
-      gboolean unused (false);
-      char * pch = g_mime_message_get_body (_message, true, &unused);
-      h = content_to_utf8 (pch, message_charset, group_charset);
-      g_free (pch);
+      h = get_utf8_body (_message);
 
       // remove the sig...
       int sig_index (0);
@@ -1463,7 +1515,7 @@ BodyPane :: create_followup_or_reply (bool is_reply)
       // trim...
       StringView v (h);
       v.trim ();
-      h = std::string(v.str,v.len);
+      h = std::string (v.str, v.len);
     }
 
     // quote the body
