@@ -67,102 +67,6 @@ namespace
   };
 }
 
-class HeaderPane::Row: public PanTreeStore::Row
-{
-  public:
-    const Article * article;
-    char * date_str;
-    Quark short_author;
-    int action;
-    int state;
-    bool is_read;
-
-  private:
-    static Quark build_short_author (const Article * article) {
-      return Quark (GNKSA :: get_short_author_name (article->author.c_str()));
-    }
-
-  private:
-    mutable char * collated_subject;
-    mutable char * collated_author;
-
-    // casefolded utf-8-friendly comparison key...
-    static char* do_collate (const StringView& view) {
-      StringView in (view);
-      while (!in.empty()) {
-	const gunichar ch = g_utf8_get_char (in.str);
-	if (!g_unichar_isalnum (ch)) // eat everything before the first alpha
-	  in.eat_chars (g_unichar_to_utf8 (ch, NULL));
-	else if (in.len>=3 // Re: reply leader
-		 && (in.str[0]=='R' ||in.str[0]=='r')
-		 && (in.str[1]=='E' || in.str[1]=='e')
-		 && in.str[2]==':')
-	  in.eat_chars (3);
-	else
-	  break;
-      }
-
-      if (in.empty())
-	return g_strdup ("");
-
-      char * casefold = g_utf8_casefold (in.str, in.len);
-      char * ret = g_utf8_collate_key (casefold, -1);
-      g_free (casefold);
-      return ret;
-    }
-
-  public:
-
-    // lazy instantation... it's expensive and user might never sort by this key
-    char* get_collated_subject () const {
-      if (!collated_subject)
-	   collated_subject = do_collate (article->subject.to_view());
-      return collated_subject;
-    }
-
-    // lazy instantation... it's expensive and user might never sort by this key
-    char* get_collated_author () const {
-      if (!collated_author)
-	   collated_author = do_collate (article->author.to_view());
-      return collated_author;
-    }
-
-  public:
-    Row (const Data& data, const Article * a, char * date_str_in, int action_in, int state_in):
-      article(a),
-      date_str (date_str_in),
-      short_author(build_short_author(a)),
-      action (action_in),
-      state (state_in),
-      is_read (data.is_read (a)),
-      collated_subject(0),
-      collated_author(0) {}
-
-    virtual ~Row () {
-      g_free (collated_subject);
-      g_free (collated_author);
-      g_free (date_str);
-    }
-
-  public:
-    virtual void get_value (int column, GValue* setme) {
-      switch (column) {
-	case COL_DATE_STR:         set_value_static_string (setme, date_str); break;
-	case COL_STATE:            set_value_int (setme, state); break;
-	case COL_ACTION:           set_value_int (setme, action); break;
-	case COL_SCORE:            set_value_int (setme, article->score); break;
-	case COL_LINES:            set_value_ulong (setme, article->get_line_count()); break;
-	case COL_BYTES:            set_value_ulong (setme, article->get_byte_count()); break;
-	case COL_DATE:             set_value_ulong (setme, (unsigned long)article->time_posted); break;
-	case COL_ARTICLE_POINTER:  set_value_pointer (setme, (void*)article); break;
-	case COL_COLLATED_SUBJECT: set_value_static_string (setme, get_collated_subject()); break;
-	case COL_COLLATED_AUTHOR:  set_value_static_string (setme, get_collated_author()); break;
-	case COL_SUBJECT:          set_value_static_string (setme, article->author.c_str()); break;
-	case COL_SHORT_AUTHOR:     set_value_static_string (setme, short_author.c_str()); break;
-    }
-  }
-};
-
 const Article*
 HeaderPane :: get_article (GtkTreeModel* model, GtkTreeIter* iter)
 {
@@ -439,11 +343,8 @@ namespace
 HeaderPane::Row*
 HeaderPane :: get_row (const Quark& message_id)
 {
-  Row * ret (0);
   mid_to_row_t::iterator it (_mid_to_row.find (message_id));
-  if (it != _mid_to_row.end())
-    ret = it->second;
-  return ret;
+  return it==_mid_to_row.end() ? 0 : *it;
 }
 
 HeaderPane::Row*
@@ -454,8 +355,10 @@ HeaderPane :: create_row (const EvolutionDateMaker & e,
   const int state (get_article_state (_data, a));
   char * date_str (e.get_date_string (a->time_posted));
   Row * row = new Row (_data, a, date_str, action, state);
-  assert (!get_row(a->message_id));
-  _mid_to_row[a->message_id] = row;
+
+  std::pair<mid_to_row_t::iterator,bool> result (_mid_to_row.insert (row));
+  g_assert (result.second);
+  
   return row;
 }
 
@@ -695,7 +598,7 @@ void
 HeaderPane :: rebuild_all_article_states ()
 {
   foreach (mid_to_row_t, _mid_to_row, it)
-    rebuild_article_state (it->first);
+    rebuild_article_state ((*it)->article->message_id);
 }
 
 void
@@ -712,48 +615,6 @@ HeaderPane :: on_group_read (const Quark& group)
 *****
 ****/
 
-namespace
-{
-  typedef std::map<Quark,quarks_t> parent_to_children_t;
-}
-
-#if 0
-struct HeaderPane::RowInserter: public PanTreeStore::WalkFunctor
-{
-  HeaderPane * header_pane;
-  Data & data;
-  Data::ArticleTree * data_tree;
-  parent_to_children_t & kids;
-  EvolutionDateMaker date_maker;
-
-  RowInserter (HeaderPane *h, Data& d, Data::ArticleTree *t, parent_to_children_t& k):
-    header_pane(h), data(d), data_tree(t), kids(k) {}
-  virtual ~RowInserter () {}
-
-  void append_children (PanTreeStore * store, GtkTreeIter * parent, const Quark& parent_mid)
-  {
-    parent_to_children_t::iterator map_it = kids.find (parent_mid);
-    if (map_it != kids.end())
-    {
-      std::vector<PanTreeStore::Row*> rows;
-      foreach_const (quarks_t, map_it->second, it) {
-	const Article * article (data_tree->get_article (*it));
-	rows.push_back (header_pane->create_row (date_maker, article));
-      }
-      store->append (store->get_row(parent), rows);
-    }
-  }
-    
-  virtual bool operator()(PanTreeStore *store, PanTreeStore::Row* r, GtkTreeIter *it, GtkTreePath *unused)
-  {
-    Row * row (dynamic_cast<Row*>(r));
-    const Article * a (row->article);
-    append_children (store, it, a->message_id);
-    return true; // keep marching
-  }
-};
-#endif
-
 namespace
 {
   struct ArticleIsNotInSet: public ArticleTester
@@ -863,8 +724,11 @@ HeaderPane :: on_tree_change (const Data::ArticleTree::Diffs& diffs)
     PanTreeStore::rows_t rows;
     rows.reserve (diffs.removed.size());
     foreach_const (quarks_t, diffs.removed, qit) {
-      rows.push_back (get_row (*qit));
-      _mid_to_row.erase (*qit);
+      mid_to_row_t::iterator it (_mid_to_row.find (*qit));
+      if (it!=_mid_to_row.end()) {
+        rows.push_back (*it);
+        _mid_to_row.erase (it);
+      }
     }
     _tree_store->remove (rows);
   }
