@@ -25,6 +25,7 @@
 #include <pan/data/article-cache.h>
 #include <pan/data/data.h>
 #include <pan/data/filter-info.h>
+#include <pan/usenet-utils/gnksa.h>
 #include <pan/tasks/queue.h>
 #include <pan/gui/action-manager.h>
 #include <pan/gui/pan-tree.h>
@@ -146,9 +147,129 @@ namespace pan
       ActionManager& _action_manager;
 
     private:
-      class Row;
-      typedef std::map<Quark,Row*> mid_to_row_t;
+
+      enum {
+        COL_DATE_STR, COL_STATE, COL_ACTION, COL_SCORE,
+        COL_LINES, COL_BYTES, COL_DATE, COL_ARTICLE_POINTER,
+        COL_COLLATED_SUBJECT, COL_COLLATED_AUTHOR, COL_SUBJECT,
+        COL_SHORT_AUTHOR, N_COLUMNS
+      };
+
+      class Row: public PanTreeStore::Row
+      {
+        public:
+          const Article * article;
+          char * date_str;
+          Quark short_author;
+          int action;
+          int state;
+          bool is_read;
+
+        private:
+          static Quark build_short_author (const Article * article) {
+            return Quark (GNKSA :: get_short_author_name (article->author.c_str()));
+          }
+
+        private:
+          mutable char * collated_subject;
+          mutable char * collated_author;
+
+          // casefolded utf-8-friendly comparison key...
+          static char* do_collate (const StringView& view) {
+            StringView in (view);
+            while (!in.empty()) {
+              const gunichar ch = g_utf8_get_char (in.str);
+              if (!g_unichar_isalnum (ch)) // eat everything before the first alpha
+                in.eat_chars (g_unichar_to_utf8 (ch, NULL));
+              else if (in.len>=3 // Re: reply leader
+                   && (in.str[0]=='R' ||in.str[0]=='r')
+                   && (in.str[1]=='E' || in.str[1]=='e')
+                   && in.str[2]==':')
+                in.eat_chars (3);
+              else
+                break;
+            }
+
+            if (in.empty())
+              return g_strdup ("");
+
+            char * casefold = g_utf8_casefold (in.str, in.len);
+            char * ret = g_utf8_collate_key (casefold, -1);
+            g_free (casefold);
+            return ret;
+          }
+
+        public:
+
+          // lazy instantation... it's expensive and user might never sort by this key
+          char* get_collated_subject () const {
+            if (!collated_subject)
+                 collated_subject = do_collate (article->subject.to_view());
+            return collated_subject;
+          }
+
+          // lazy instantation... it's expensive and user might never sort by this key
+          char* get_collated_author () const {
+            if (!collated_author)
+                 collated_author = do_collate (article->author.to_view());
+            return collated_author;
+          }
+
+        public:
+          Row (const Data& data, const Article * a, char * date_str_in, int action_in, int state_in):
+            article(a),
+            date_str (date_str_in),
+            short_author(build_short_author(a)),
+            action (action_in),
+            state (state_in),
+            is_read (data.is_read (a)),
+            collated_subject(0),
+            collated_author(0) {}
+
+          virtual ~Row () {
+            g_free (collated_subject);
+            g_free (collated_author);
+            g_free (date_str);
+          }
+
+        public:
+          virtual void get_value (int column, GValue* setme) {
+            switch (column) {
+              case COL_DATE_STR:         set_value_static_string (setme, date_str); break;
+              case COL_STATE:            set_value_int (setme, state); break;
+              case COL_ACTION:           set_value_int (setme, action); break;
+              case COL_SCORE:            set_value_int (setme, article->score); break;
+              case COL_LINES:            set_value_ulong (setme, article->get_line_count()); break;
+              case COL_BYTES:            set_value_ulong (setme, article->get_byte_count()); break;
+              case COL_DATE:             set_value_ulong (setme, (unsigned long)article->time_posted); break;
+              case COL_ARTICLE_POINTER:  set_value_pointer (setme, (void*)article); break;
+              case COL_COLLATED_SUBJECT: set_value_static_string (setme, get_collated_subject()); break;
+              case COL_COLLATED_AUTHOR:  set_value_static_string (setme, get_collated_author()); break;
+              case COL_SUBJECT:          set_value_static_string (setme, article->author.c_str()); break;
+              case COL_SHORT_AUTHOR:     set_value_static_string (setme, short_author.c_str()); break;
+            }
+          }
+      };
+
+      struct RowLessThan
+      {
+        bool operator () (const Row* a, const Row* b) const {
+          return a->article->message_id < b->article->message_id;
+        }
+        bool operator () (const Row* a, const Quark& b) const {
+          return a->article->message_id < b;
+        }
+        bool operator () (const Quark& a, const Row* b) const {
+          return a < b->article->message_id;
+        }
+      };
+
+      typedef sorted_vector <Row*, true, RowLessThan> mid_to_row_t;
+
       mid_to_row_t _mid_to_row;
+
+    private:
+
       static const Article* get_article (GtkTreeModel*, GtkTreeIter*);
       int find_highest_followup_score (GtkTreeModel*, GtkTreeIter*) const;
       Row* create_row (const EvolutionDateMaker&, const Article*);
