@@ -39,8 +39,13 @@ namespace pan
   class PostUI: private Progress::Listener
   {
     public:
-      PostUI (Data&, Queue&, GroupServer&, Profiles&, GMimeMessage* message,
-              Prefs&, GroupPrefs&);
+      static PostUI* create_window (GtkWindow*, Data&, Queue&, GroupServer&, Profiles&,
+                                    GMimeMessage*, Prefs&, GroupPrefs&);
+    
+    protected:
+      PostUI (GtkWindow*, Data&, Queue&, GroupServer&, Profiles&,
+              GMimeMessage*, Prefs&, GroupPrefs&);
+    public:
       ~PostUI ();
 
     public:
