@@ -78,6 +78,10 @@ namespace
   {
     delete static_cast<SaveDialog*>(castme);
   }
+
+  enum { PATH_GROUP, PATH_ENTRY };
+
+  int path_mode (PATH_GROUP);
 }
 
 void
@@ -89,16 +93,19 @@ SaveDialog :: response_cb (GtkDialog * dialog,
   {
     SaveDialog * self (static_cast<SaveDialog*>(user_data));
 
+    // set the path mode based on what widgets exist & are set
+    GtkWidget * gr (self->_save_group_path_radio);
+    GtkWidget * er (self->_save_custom_path_radio);
+    if (gr && er)
+      path_mode = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(gr)) ? PATH_GROUP : PATH_ENTRY;
+    else
+      path_mode = PATH_ENTRY;
+
     // get the save path
     std::string path;
-    bool use_custom_path (true);
-    GtkWidget * w (self->_save_group_path_radio);
-    if (w && gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w))) {
-      path = (const char*) g_object_get_data (G_OBJECT(w), "default-group-save-path");
-      use_custom_path = path.empty();
-    }
-    if (use_custom_path) {
-      // get the save path...
+    if (path_mode == PATH_GROUP)
+      path = (const char*) g_object_get_data (G_OBJECT(gr), "default-group-save-path");
+    else if (path_mode == PATH_ENTRY)  {
       path = pan :: file_entry_get (self->_save_path_entry);
       self->_prefs.set_string ("default-save-attachments-path", path);
     }
@@ -195,11 +202,14 @@ SaveDialog :: SaveDialog (Prefs                       & prefs,
       HIG::workarea_add_section_spacer (t, row, 2);
   }
 
+    if (path_mode==PATH_GROUP && !have_group_default)
+      path_mode = PATH_ENTRY;
+
     h = gtk_hbox_new (FALSE, 0);
     if (have_group_default) {
       w = _save_custom_path_radio = gtk_radio_button_new_with_mnemonic (NULL, _("C_ustom path:"));
       gtk_box_pack_start (GTK_BOX(h), w, false, false, 0);
-      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(w), true);
+      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(w), path_mode==PATH_ENTRY);
     }
     w = _save_path_entry = file_entry_new (_("Save Files to Path"));
     gtk_box_pack_start (GTK_BOX(h), w, true, true, 0);
@@ -220,6 +230,7 @@ SaveDialog :: SaveDialog (Prefs                       & prefs,
       char * pch = g_strdup_printf (_("Group's _default path: %s"), group_path.c_str());
       w = gtk_radio_button_new_with_mnemonic_from_widget (GTK_RADIO_BUTTON(_save_custom_path_radio), pch);
       _save_group_path_radio = w;
+      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(w), path_mode==PATH_GROUP);
       g_object_set_data_full (G_OBJECT(w), "default-group-save-path", g_strdup(group_path.c_str()), g_free);
       g_free (pch);
       HIG :: workarea_add_wide_control (t, &row, w);
