@@ -112,13 +112,8 @@ NNTP :: fire_done_func (Health health)
 ****  WRITING
 ***/
 
-void
-NNTP :: onSocketOpen (Socket * sock)
-{
-}
-
 bool
-NNTP :: onSocketResponse (Socket * sock, const StringView& line_in)
+NNTP :: on_socket_response (Socket * sock, const StringView& line_in)
 {
    enum State { CMD_FAIL, CMD_DONE, CMD_MORE, CMD_NEXT, CMD_RETRY };
    State state;
@@ -264,13 +259,13 @@ NNTP :: onSocketResponse (Socket * sock, const StringView& line_in)
 }
 
 void
-NNTP :: onSocketAbort (Socket * sock)
+NNTP :: on_socket_abort (Socket * sock)
 {
    fire_done_func (FAIL);
 }
 
 void
-NNTP :: onSocketError (Socket * sock)
+NNTP :: on_socket_error (Socket * sock)
 {
    _socket_error = true;
    fire_done_func (RETRY);
