@@ -65,9 +65,9 @@ TaskArticle :: TaskArticle (const ServerRank          & server_rank,
                             ArticleRead               & read,
                             Task::Listener            * listener,
                             SaveMode                    save_mode,
-                            const StringView          & save_path):
+                            const Quark               & save_path):
   Task (save_path.empty() ? "BODIES" : "SAVE", get_description (article, !save_path.empty())),
-  _save_path (save_path.str, save_path.len),
+  _save_path (save_path),
   _server_rank (server_rank),
   _cache (cache),
   _read (read),
@@ -291,7 +291,7 @@ TaskArticle :: on_finished ()
       gsize length (0);
       if (g_file_get_contents (it->c_str(), &contents, &length, NULL) && length>0)
       {
-        file :: ensure_dir_exists (_save_path);
+        file :: ensure_dir_exists (_save_path.c_str());
         gchar * basename (g_path_get_basename (it->c_str()));
         gchar * filename (g_build_filename (_save_path.c_str(), basename, NULL));
         FILE * fp = fopen (filename, "w+");
@@ -335,7 +335,7 @@ TaskArticle :: on_finished ()
       {
         // make sure the directory exists...
         if (!_save_path.empty())
-          file :: ensure_dir_exists (_save_path);
+          file :: ensure_dir_exists (_save_path.c_str());
 
         // find a unique filename...
         char * fname (0);
