@@ -47,10 +47,10 @@ namespace pan
                    ArticleRead        & read,
                    Task::Listener     * l=0,
                    SaveMode             save_mode = NONE,
-                   const StringView   & save_path = StringView());
+                   const Quark        & save_path = Quark());
       virtual ~TaskArticle ();
       time_t get_time_posted () const { return _time_posted; }
-      const std::string& get_save_path () const { return _save_path; }
+      const Quark& get_save_path () const { return _save_path; }
       const Article& get_article () const { return _article; }
         
     public: // Task subclass
@@ -64,7 +64,7 @@ namespace pan
       virtual void on_nntp_done  (NNTP*, Health);
 
     protected:
-      const std::string _save_path;
+      const Quark _save_path;
       const ServerRank& _server_rank;
       ArticleCache& _cache;
       ArticleRead& _read;
