@@ -80,14 +80,14 @@ TaskGroups :: on_nntp_line (NNTP               * nntp,
                             const StringView   & line)
 {
   char permission ('?');
-  char * name (0);
-  char * description (0);
+  std::string name_str;
+  std::string desc_str;
 
   if (_step == LIST)
   {
     StringView myline(line), tmp, post;
     myline.pop_token (tmp);
-    name = pan :: content_to_utf8 (tmp.str, tmp.len);
+    name_str = content_to_utf8 (tmp);
     myline.pop_token (tmp); // skip low number
     myline.pop_token (tmp); // skip high number
     myline.pop_token (post); // ok to post?
@@ -98,21 +98,20 @@ TaskGroups :: on_nntp_line (NNTP               * nntp,
   {
     const char * pch = line.begin();
     while (pch!=line.end() && !isspace((int)*pch)) ++pch;
-    name = pan :: content_to_utf8 (line.str, pch-line.str);
-    description = pan :: content_to_utf8 (pch, line.str+line.len-pch);
+    name_str = content_to_utf8 (StringView (line.str, pch-line.str));
+    desc_str = content_to_utf8 (StringView (pch, line.str+line.len-pch));
   }
 
-  if (name)
-    g_strstrip (name);
-  if (description)
-    g_strstrip (description);
+  StringView name (name_str);
+  StringView desc (desc_str);
+  name.trim ();
+  desc.trim ();
 
-  if (name && *name){
+  if (!name.empty()) {
     const Quark name_quark (name);
     Data::NewGroup& ng (_new_groups[name_quark]);
     ng.group = name_quark;
-    if (description)
-      ng.description = description;
+    ng.description.assign (desc.str, desc.len);
     if (permission != '?')
       ng.permission = permission;
   }
@@ -124,8 +123,6 @@ TaskGroups :: on_nntp_line (NNTP               * nntp,
   }
 
   increment_step ();
-  g_free (description);
-  g_free (name);
 }
 
 void
